@@ -1,5 +1,5 @@
 // -*- C++ -*-
-// Copyright (C) 2008-2011 Red Hat Inc.
+// Copyright (C) 2008-2014 Red Hat Inc.
 //
 // This file is part of systemtap, and is free software.  You can
 // redistribute it and/or modify it under the terms of the GNU General
@@ -18,7 +18,8 @@
 
 using namespace std;
 
-void dwfl_assert(const string& desc, int rc)
+void dwfl_assert(const string& desc, int rc,
+                 const string& file, int line)
 {
   if (rc == 0)
     return;
@@ -27,10 +28,11 @@ void dwfl_assert(const string& desc, int rc)
     msg += (dwfl_errmsg (rc) ?: "?");
   else
     msg += std::strerror (rc);
-  throw semantic_error (msg);
+  throw semantic_error (file+":"+lex_cast(line), msg);
 }
 
-void dwarf_assert(const string& desc, int rc)
+void dwarf_assert(const string& desc, int rc,
+                  const string& file, int line)
 {
   if (rc == 0)
     return;
@@ -39,7 +41,7 @@ void dwarf_assert(const string& desc, int rc)
     msg += dwarf_errmsg (rc);
   else
     msg += std::strerror (rc);
-  throw semantic_error (msg);
+  throw semantic_error (file+":"+lex_cast(line), msg);
 }
 
 
@@ -115,6 +117,7 @@ dwarf_type_name(Dwarf_Die *type_die, ostream& o)
     case DW_TAG_array_type:
     case DW_TAG_const_type:
     case DW_TAG_volatile_type:
+    case DW_TAG_restrict_type:
       done = false;
       break;
 
@@ -177,6 +180,9 @@ dwarf_type_name(Dwarf_Die *type_die, ostream& o)
     case DW_TAG_volatile_type:
       o << " volatile";
       break;
+    case DW_TAG_restrict_type:
+      o << " restrict";
+      break;
     default:
       return false;
     }
