@@ -1,5 +1,5 @@
 // C++ interface to dwfl
-// Copyright (C) 2005-2014 Red Hat Inc.
+// Copyright (C) 2005-2015 Red Hat Inc.
 // Copyright (C) 2005-2007 Intel Corporation.
 // Copyright (C) 2008 James.Bottomley@HansenPartnership.com
 //
@@ -17,6 +17,7 @@
 #include "session.h"
 #include "unordered.h"
 #include "setupdwfl.h"
+#include "stringtable.h"
 
 #include <cstring>
 #include <iostream>
@@ -25,6 +26,12 @@
 #include <string>
 #include <vector>
 
+// Old elf.h doesn't know about this machine type.
+#ifndef EM_AARCH64
+#define EM_AARCH64 183
+#endif
+
+
 extern "C" {
 #include <elfutils/libdwfl.h>
 #include <regex.h>
@@ -38,7 +45,7 @@ struct symbol_table;
 struct base_query;
 struct external_function_query;
 
-enum lineno_t { ABSOLUTE, RELATIVE, RANGE, WILDCARD };
+enum lineno_t { ABSOLUTE, RELATIVE, WILDCARD, ENUMERATED };
 enum info_status { info_unknown, info_present, info_absent };
 
 // module -> cu die[]
@@ -55,7 +62,7 @@ typedef unordered_map<std::string, Dwarf_Die> cu_type_cache_t;
 typedef unordered_map<void*, cu_type_cache_t*> mod_cu_type_cache_t;
 
 // function -> die
-typedef unordered_multimap<std::string, Dwarf_Die> cu_function_cache_t;
+typedef unordered_multimap<interned_string, Dwarf_Die> cu_function_cache_t;
 typedef std::pair<cu_function_cache_t::iterator,
                   cu_function_cache_t::iterator>
         cu_function_cache_range_t;
@@ -104,11 +111,11 @@ module_info
   info_status dwarf_status;     // module has dwarf info?
   info_status symtab_status;    // symbol table cached?
 
-  std::set<std::string> inlined_funcs;
-  std::set<std::string> plt_funcs;
+  std::set<interned_string> inlined_funcs;
+  std::set<interned_string> plt_funcs;
   std::set<std::pair<std::string,std::string> > marks; /* <provider,name> */
 
-  void get_symtab(base_query *q);
+  void get_symtab();
   void update_symtab(cu_function_cache_t *funcs);
 
   module_info(const char *name) :
@@ -140,12 +147,12 @@ module_cache
 struct base_func_info
 {
   base_func_info()
-    : decl_file(NULL), decl_line(-1), entrypc(0)
+  : decl_line(-1), entrypc(0)
   {
     std::memset(&die, 0, sizeof(die));
   }
-  std::string name;
-  char const * decl_file;
+  interned_string name;
+  interned_string decl_file;
   int decl_line;
   Dwarf_Die die;
   Dwarf_Addr entrypc;
@@ -167,6 +174,17 @@ struct inline_instance_info : base_func_info
   bool operator<(const inline_instance_info& other) const;
 };
 
+/* We'll need some context when dwflpp::loc2c_error is called.
+   So we can attach some detailed information about (the location of)
+   the DWARF that failed to be translated. Needs updating before each
+   loc2c c_translate call. */
+struct loc2c_context
+{
+  Dwarf_Die *die;
+  Dwarf_Addr pc;
+public:
+  loc2c_context(): die(0), pc(0) {}
+};
 
 struct dwflpp
 {
@@ -186,6 +204,10 @@ struct dwflpp
   std::string module_name;
   std::string function_name;
 
+  // Some context for dwflpp::loc2c_error callback.
+  // Needs updating before each loc2c c_translate call.
+  struct loc2c_context l2c_ctx;
+
   dwflpp(systemtap_session & session, const std::string& user_module, bool kernel_p);
   dwflpp(systemtap_session & session, const std::vector<std::string>& user_modules, bool kernel_p);
   ~dwflpp();
@@ -283,11 +305,15 @@ struct dwflpp
 
   template<typename T>
   int iterate_over_notes (T *object,
-			  void (* callback)(T*, int, const char*, size_t))
+                          void (* callback)(T*, const std::string&,
+                                                const std::string&,
+                                                int, const char*, size_t))
     {
       // See comment block in iterate_over_modules()
       return iterate_over_notes<void>((void*)object,
                                       (void (*)(void*,
+                                                const std::string&,
+                                                const std::string&,
                                                 int,
                                                 const char*,
                                                 size_t))callback);
@@ -314,11 +340,12 @@ struct dwflpp
 
   template<typename T>
   void iterate_over_srcfile_lines (char const * srcfile,
-                                   int linenos[2],
+                                   const std::vector<int>& linenos,
                                    enum lineno_t lineno_type,
                                    base_func_info_map_t& funcs,
                                    void (*callback) (Dwarf_Addr,
                                                      int, T*),
+                                   bool has_nearest,
                                    T *data)
     {
       // See comment block in iterate_over_modules()
@@ -328,15 +355,18 @@ struct dwflpp
                                        funcs,
                                        (void (*)(Dwarf_Addr,
                                                  int, void*))callback,
+                                       has_nearest,
                                        (void*)data);
     }
 
   template<typename T>
   void iterate_over_labels (Dwarf_Die *begin_die,
                             const std::string& sym,
-                            const std::string& function,
+                            const base_func_info& function,
+                            const std::vector<int>& linenos,
+                            enum lineno_t lineno_type,
                             T *data,
-                            void (* callback)(const std::string&,
+                            void (* callback)(const base_func_info&,
                                               const char*,
                                               const char*,
                                               int,
@@ -348,8 +378,10 @@ struct dwflpp
       iterate_over_labels<void>(begin_die,
                                 sym,
                                 function,
+                                linenos,
+                                lineno_type,
                                 (void*)data,
-                                (void (*)(const std::string&,
+                                (void (*)(const base_func_info&,
                                           const char*,
                                           const char*,
                                           int,
@@ -407,7 +439,7 @@ struct dwflpp
                                       std::string const & local,
                                       const target_symbol *e,
                                       bool lvalue,
-                                      exp_type & ty);
+                                      Dwarf_Die *die_mem);
   Dwarf_Die* type_die_for_local (std::vector<Dwarf_Die>& scopes,
                                  Dwarf_Addr pc,
                                  std::string const & local,
@@ -418,7 +450,7 @@ struct dwflpp
                                        Dwarf_Addr pc,
                                        const target_symbol *e,
                                        bool lvalue,
-                                       exp_type & ty);
+                                       Dwarf_Die *die_mem);
   Dwarf_Die* type_die_for_return (Dwarf_Die *scope_die,
                                   Dwarf_Addr pc,
                                   const target_symbol *e,
@@ -427,19 +459,28 @@ struct dwflpp
   std::string literal_stmt_for_pointer (Dwarf_Die *type_die,
                                         const target_symbol *e,
                                         bool lvalue,
-                                        exp_type & ty);
+                                        Dwarf_Die *die_mem);
   Dwarf_Die* type_die_for_pointer (Dwarf_Die *type_die,
                                    const target_symbol *e,
                                    Dwarf_Die *die_mem);
 
-  bool blacklisted_p(const std::string& funcname,
-                     const std::string& filename,
-                     int line,
-                     const std::string& module,
-                     Dwarf_Addr addr,
-                     bool has_return);
+  enum blacklisted_type
+    {  blacklisted_none, // not blacklisted
+       blacklisted_section,
+       blacklisted_kprobes,
+       blacklisted_function,
+       blacklisted_function_return,
+       blacklisted_file
+    };
 
-  Dwarf_Addr relocate_address(Dwarf_Addr addr, std::string& reloc_section);
+  blacklisted_type blacklisted_p(interned_string funcname,
+                                 interned_string filename,
+                                 int line,
+                                 interned_string module,
+                                 Dwarf_Addr addr,
+                                 bool has_return);
+
+  Dwarf_Addr relocate_address(Dwarf_Addr addr, interned_string& reloc_section);
 
   void resolve_unqualified_inner_typedie (Dwarf_Die *typedie,
                                           Dwarf_Die *innerdie,
@@ -539,6 +580,12 @@ private:
   void collect_all_lines(char const * srcfile,
                          base_func_info_map_t& funcs,
                          lines_t& matching_lines);
+  std::pair<int,int> get_nearest_linenos(char const * srcfile,
+                                         int lineno,
+                                         base_func_info_map_t& funcs);
+  int get_nearest_lineno(char const * srcfile,
+                         int lineno,
+                         base_func_info_map_t& funcs);
   void suggest_alternative_linenos(char const * srcfile,
                                    int lineno,
                                    base_func_info_map_t& funcs);
@@ -566,6 +613,13 @@ private:
                                                  Dwarf_Die *vardie,
                                                  Dwarf_Attribute *fb_attr_mem);
 
+  std::string die_location_as_string(Dwarf_Addr, Dwarf_Die*);
+  std::string die_location_as_function_string(Dwarf_Addr, Dwarf_Die*);
+  std::string pc_die_line_string(Dwarf_Addr, Dwarf_Die*);
+
+  /* source file name, line and column info for pc in current cu. */
+  const char *pc_line (Dwarf_Addr, int *, int *);
+
   struct location *translate_location(struct obstack *pool,
                                       Dwarf_Attribute *attr,
                                       Dwarf_Die *die,
@@ -597,7 +651,7 @@ private:
                                        const target_symbol *e,
                                        std::string &,
                                        std::string &,
-                                       exp_type & ty);
+                                       Dwarf_Die *enddie);
 
   std::string express_as_string (std::string prelude,
                                  std::string postlude,
@@ -670,6 +724,8 @@ dwflpp::iterate_over_types<void>(Dwarf_Die *top_die,
                                  void *data);
 template<> int
 dwflpp::iterate_over_notes<void>(void *object, void (*callback)(void*,
+                                                                const std::string&,
+                                                                const std::string&,
                                                                 int,
                                                                 const char*,
                                                                 size_t));
@@ -682,18 +738,21 @@ dwflpp::iterate_over_plt<void>(void *object, void (*callback)(void*,
                                                               size_t));
 template<> void
 dwflpp::iterate_over_srcfile_lines<void>(char const * srcfile,
-                                         int linenos[2],
+                                         const std::vector<int>& linenos,
                                          enum lineno_t lineno_type,
                                          base_func_info_map_t& funcs,
                                          void (* callback) (Dwarf_Addr,
                                                             int, void*),
+                                         bool has_nearest,
                                          void *data);
 template<> void
 dwflpp::iterate_over_labels<void>(Dwarf_Die *begin_die,
                                   const std::string& sym,
-                                  const std::string& function,
+                                  const base_func_info& function,
+                                  const std::vector<int>& linenos,
+                                  enum lineno_t lineno_type,
                                   void *data,
-                                  void (* callback)(const std::string&,
+                                  void (* callback)(const base_func_info&,
                                                     const char*,
                                                     const char*,
                                                     int,
