@@ -224,14 +224,14 @@ find_script_hash (systemtap_session& s, const string& script)
   // Hash user-specified arguments (that change the generated module).
   h.add("Bulk Mode (-b): ", s.bulk_mode);
   h.add("Timing (-t): ", s.timing);
-  h.add("Prologue Searching (-P): ", s.prologue_searching);
   h.add("Skip Badvars (--skip-badvars): ", s.skip_badvars);
   h.add("Privilege (--privilege): ", s.privilege);
   h.add("Compatible (--compatible): ", s.compatible);
   h.add("Omit Werror (undocumented): ", s.omit_werror);
-  h.add("Prologue Searching (-P): ", s.prologue_searching);
   h.add("Error suppression (--suppress-handler-errors): ", s.suppress_handler_errors);
   h.add("Suppress Time Limits (--suppress-time-limits): ", s.suppress_time_limits);
+  h.add("Prologue Searching (--prologue-searching[=WHEN]): ", int(s.prologue_searching_mode));
+
   for (unsigned i = 0; i < s.c_macros.size(); i++)
     h.add("Macros: ", s.c_macros[i]);
 
