@@ -6,6 +6,7 @@
 static void *_stp_shm_base;
 static void *_stp_shm_alloc(size_t size);
 
+#include "session_attributes.h"
 #include "transport.h"
 
 // Global state shared throughout the module
@@ -21,6 +22,8 @@ struct stp_runtime_session {
 
         unsigned long _hash_seed;
 
+	struct _stp_session_attributes _session_attributes;
+
 #ifdef STP_ALIBI
 	atomic_t _probe_alibi[STP_PROBE_COUNT];
 #endif
@@ -76,6 +79,14 @@ static inline unsigned long _stap_hash_seed()
 }
 
 
+static inline struct _stp_session_attributes *stp_session_attributes(void)
+{
+	if (likely(_stp_session()))
+		return &_stp_session()->_session_attributes;
+	return NULL;
+}
+
+
 #ifdef STP_ALIBI
 static inline atomic_t *probe_alibi(size_t index)
 {
@@ -184,6 +195,8 @@ static int stp_session_init(void)
 
 	_stp_session()->_hash_seed = _stp_random_u ((unsigned long)-1);
 
+	stp_session_attributes_init();
+
 #ifdef STP_ALIBI
 	// Initialize all the alibi counters
 	for (i = 0; i < STP_PROBE_COUNT; ++i)
