@@ -1,7 +1,7 @@
 /*
  * utrace infrastructure interface for debugging user processes
  *
- * Copyright (C) 2006-2012 Red Hat, Inc.  All rights reserved.
+ * Copyright (C) 2006-2014 Red Hat, Inc.  All rights reserved.
  *
  * This copyrighted material is made available to anyone wishing to use,
  * modify, copy, or redistribute it subject to the terms and conditions
@@ -22,10 +22,12 @@
 #include <linux/sched.h>
 #include <linux/freezer.h>
 #include <linux/slab.h>
-#include <linux/spinlock.h>
 #include <trace/events/sched.h>
 #include <trace/events/syscalls.h>
 #include "stp_task_work.c"
+#include "linux/stp_tracepoint.h"
+
+#include "stp_helper_lock.h"
 
 /*
  * Per-thread structure private to utrace implementation.
@@ -55,7 +57,7 @@
  * in time to have their callbacks seen.
  */
 struct utrace {
-	spinlock_t lock;
+	stp_spinlock_t lock;
 	struct list_head attached, attaching;
 
 	struct utrace_engine *reporting;
@@ -65,7 +67,9 @@ struct utrace {
 	unsigned int death:1;	/* in utrace_report_death() now */
 	unsigned int reap:1;	/* release_task() has run */
 	unsigned int pending_attach:1; /* need splice_attaching() */
-	unsigned int task_work_added:1; /* called task_work_add() */
+	unsigned int task_work_added:1; /* called task_work_add() on 'work' */
+	unsigned int report_work_added:1; /* called task_work_add()
+					   * on 'report_work' */
 
 	unsigned long utrace_flags;
 
@@ -73,6 +77,7 @@ struct utrace {
 	struct task_struct *task;
 
 	struct task_work work;
+	struct task_work report_work;
 };
 
 #define TASK_UTRACE_HASH_BITS 5
@@ -80,7 +85,7 @@ struct utrace {
 
 static struct hlist_head task_utrace_table[TASK_UTRACE_TABLE_SIZE];
 //DEFINE_MUTEX(task_utrace_mutex);      /* Protects task_utrace_table */
-static DEFINE_SPINLOCK(task_utrace_lock); /* Protects task_utrace_table */
+static STP_DEFINE_SPINLOCK(task_utrace_lock); /* Protects task_utrace_table */
 
 static struct kmem_cache *utrace_cachep;
 static struct kmem_cache *utrace_engine_cachep;
@@ -105,6 +110,22 @@ static void utrace_report_exec(void *cb_data __attribute__ ((unused)),
 #define __UTRACE_REGISTERED	1
 static atomic_t utrace_state = ATOMIC_INIT(__UTRACE_UNREGISTERED);
 
+// If wake_up_state() is exported, use it.
+#if defined(STAPCONF_WAKE_UP_STATE_EXPORTED)
+#define stp_wake_up_state wake_up_state
+// Otherwise, try to use try_to_wake_up(). The wake_up_state()
+// function is just a wrapper around try_to_wake_up().
+#elif defined(STAPCONF_TRY_TO_WAKE_UP_EXPORTED)
+static inline int stp_wake_up_state(struct task_struct *p, unsigned int state)
+{
+	return try_to_wake_up(p, state, 0);
+}
+// Otherwise, we'll have to look up wake_up_state() with kallsyms.
+#else
+typedef typeof(&wake_up_state) wake_up_state_fn;
+#define stp_wake_up_state (* (wake_up_state_fn)kallsyms_wake_up_state)
+#endif
+
 #if !defined(STAPCONF_SIGNAL_WAKE_UP_STATE_EXPORTED)
 // Sigh. On kernel's without signal_wake_up_state(), there is no
 // declaration to use in 'typeof(&signal_wake_up_state)'. So, we'll
@@ -184,6 +205,14 @@ static int utrace_init(void)
 		INIT_HLIST_HEAD(&task_utrace_table[i]);
 	}
 
+#if !defined(STAPCONF_TRY_TO_WAKE_UP_EXPORTED) \
+    && !defined(STAPCONF_WAKE_UP_STATE_EXPORTED)
+	kallsyms_wake_up_state = (void *)kallsyms_lookup_name("wake_up_state");
+        if (kallsyms_wake_up_state == NULL) {
+		_stp_error("Can't resolve wake_up_state!");
+		goto error;
+        }
+#endif
 #if !defined(STAPCONF_SIGNAL_WAKE_UP_STATE_EXPORTED)
 	/* The signal_wake_up_state() function (which replaces
 	 * signal_wake_up() in newer kernels) isn't exported. Look up
@@ -232,28 +261,28 @@ static int utrace_init(void)
 	if (unlikely(!utrace_engine_cachep))
 		goto error;
 
-	rc = register_trace_sched_process_fork(utrace_report_clone, NULL);
+	rc = STP_TRACE_REGISTER(sched_process_fork, utrace_report_clone);
 	if (unlikely(rc != 0)) {
 		_stp_error("register_trace_sched_process_fork failed: %d", rc);
 		goto error;
 	}
-	rc = register_trace_sched_process_exit(utrace_report_death, NULL);
+	rc = STP_TRACE_REGISTER(sched_process_exit, utrace_report_death);
 	if (unlikely(rc != 0)) {
 		_stp_error("register_trace_sched_process_exit failed: %d", rc);
 		goto error2;
 	}
-	rc = register_trace_sys_enter(utrace_report_syscall_entry, NULL);
+	rc = STP_TRACE_REGISTER(sys_enter, utrace_report_syscall_entry);
 	if (unlikely(rc != 0)) {
 		_stp_error("register_trace_sys_enter failed: %d", rc);
 		goto error3;
 	}
-	rc = register_trace_sys_exit(utrace_report_syscall_exit, NULL);
+	rc = STP_TRACE_REGISTER(sys_exit, utrace_report_syscall_exit);
 	if (unlikely(rc != 0)) {
 		_stp_error("register_trace_sys_exit failed: %d", rc);
 		goto error4;
 	}
 
-	rc = register_trace_sched_process_exec(utrace_report_exec, NULL);
+	rc = STP_TRACE_REGISTER(sched_process_exec, utrace_report_exec);
 	if (unlikely(rc != 0)) {
 		_stp_error("register_sched_process_exec failed: %d", rc);
 		goto error5;
@@ -263,36 +292,80 @@ static int utrace_init(void)
 	return 0;
 
 error5:
-	unregister_trace_sys_exit(utrace_report_syscall_exit, NULL);
+	STP_TRACE_UNREGISTER(sys_exit, utrace_report_syscall_exit);
 error4:
-	unregister_trace_sys_enter(utrace_report_syscall_entry, NULL);
+	STP_TRACE_UNREGISTER(sys_enter, utrace_report_syscall_entry);
 error3:
-	unregister_trace_sched_process_exit(utrace_report_death, NULL);
+	STP_TRACE_UNREGISTER(sched_process_exit, utrace_report_death);
 error2:
-	unregister_trace_sched_process_fork(utrace_report_clone, NULL);
+	STP_TRACE_UNREGISTER(sched_process_fork, utrace_report_clone);
 	tracepoint_synchronize_unregister();
 error:
-	if (utrace_cachep)
+	if (utrace_cachep) {
 		kmem_cache_destroy(utrace_cachep);
-	if (utrace_engine_cachep)
+		utrace_cachep = NULL;
+	}
+	if (utrace_engine_cachep) {
 		kmem_cache_destroy(utrace_engine_cachep);
+		utrace_engine_cachep = NULL;
+	}
 	return rc;
 }
 
 static int utrace_exit(void)
 {
+#ifdef STP_TF_DEBUG
+	printk(KERN_ERR "%s:%d - entry\n", __FUNCTION__, __LINE__);
+#endif
 	utrace_shutdown();
-
-	if (utrace_cachep)
-		kmem_cache_destroy(utrace_cachep);
-	if (utrace_engine_cachep)
-		kmem_cache_destroy(utrace_engine_cachep);
-
 	stp_task_work_exit();
+
+	/* After utrace_shutdown() and stp_task_work_exit() (and the
+	 * code in stap_stop_task_finder()), we're *sure* there are no
+	 * tracepoint probes or task work items running or scheduled
+	 * to be run. So, now would be a great time to actually free
+	 * everything. */
+
+	if (utrace_cachep) {
+		kmem_cache_destroy(utrace_cachep);
+		utrace_cachep = NULL;
+	}
+	if (utrace_engine_cachep) {
+		kmem_cache_destroy(utrace_engine_cachep);
+		utrace_engine_cachep = NULL;
+	}
+
+#ifdef STP_TF_DEBUG
+	printk(KERN_ERR "%s:%d - exit\n", __FUNCTION__, __LINE__);
+#endif
 	return 0;
 }
 
+/*
+ * stp_task_notify_resume() is our version of
+ * set_notify_resume(). When called, the task_work infrastructure will
+ * cause utrace_resume() to get called.
+ */
+static void
+stp_task_notify_resume(struct task_struct *target, struct utrace *utrace)
+{
+	if (! utrace->task_work_added) {
+		int rc = stp_task_work_add(target, &utrace->work);
+		if (rc == 0) {
+			utrace->task_work_added = 1;
+		}
+		/* stp_task_work_add() returns -ESRCH if the task has
+		 * already passed exit_task_work(). Just ignore this
+		 * error. */
+		else if (rc != -ESRCH) {
+			printk(KERN_ERR "%s:%d - task_work_add() returned %d\n",
+			       __FUNCTION__, __LINE__, rc);
+		}
+	}
+}
+
 static void utrace_resume(struct task_work *work);
+static void utrace_report_work(struct task_work *work);
 
 /*
  * Clean up everything associated with @task.utrace.
@@ -307,7 +380,7 @@ static void utrace_cleanup(struct utrace *utrace)
 
 	/* Free engines associated with the struct utrace, starting
 	 * with the 'attached' list then doing the 'attaching' list. */
-	spin_lock(&utrace->lock);
+	stp_spin_lock(&utrace->lock);
 	list_for_each_entry_safe(engine, next, &utrace->attached, entry) {
 #ifdef STP_TF_DEBUG
 	    printk(KERN_ERR "%s:%d - removing engine\n",
@@ -323,15 +396,32 @@ static void utrace_cleanup(struct utrace *utrace)
 	}
 
 	if (utrace->task_work_added) {
+#ifdef STP_TF_DEBUG
 		if (stp_task_work_cancel(utrace->task, &utrace_resume) == NULL)
 			printk(KERN_ERR "%s:%d - task_work_cancel() failed? task %p, %d, %s\n",
 			       __FUNCTION__, __LINE__, utrace->task,
 			       utrace->task->tgid,
 			       (utrace->task->comm ? utrace->task->comm
 				: "UNKNOWN"));
+#else
+		stp_task_work_cancel(utrace->task, &utrace_resume);
+#endif
 		utrace->task_work_added = 0;
 	}
-	spin_unlock(&utrace->lock);
+	if (utrace->report_work_added) {
+#ifdef STP_TF_DEBUG
+		if (stp_task_work_cancel(utrace->task, &utrace_report_work) == NULL)
+			printk(KERN_ERR "%s:%d - task_work_cancel() failed? task %p, %d, %s\n",
+			       __FUNCTION__, __LINE__, utrace->task,
+			       utrace->task->tgid,
+			       (utrace->task->comm ? utrace->task->comm
+				: "UNKNOWN"));
+#else
+		stp_task_work_cancel(utrace->task, &utrace_report_work);
+#endif
+		utrace->report_work_added = 0;
+	}
+	stp_spin_unlock(&utrace->lock);
 
 	/* Free the struct utrace itself. */
 	kmem_cache_free(utrace_cachep, utrace);
@@ -347,35 +437,50 @@ static void utrace_shutdown(void)
 	struct hlist_head *head;
 	struct hlist_node *node, *node2;
 
-	if (atomic_dec_and_test(&utrace_state) != __UTRACE_UNREGISTERED)
+	if (atomic_read(&utrace_state) != __UTRACE_REGISTERED)
 		return;
+	atomic_set(&utrace_state, __UTRACE_UNREGISTERED);
 
 #ifdef STP_TF_DEBUG
 	printk(KERN_ERR "%s:%d entry\n", __FUNCTION__, __LINE__);
 #endif
-	unregister_trace_sched_process_exec(utrace_report_exec, NULL);
-	unregister_trace_sched_process_fork(utrace_report_clone, NULL);
-	unregister_trace_sched_process_exit(utrace_report_death, NULL);
-	unregister_trace_sys_enter(utrace_report_syscall_entry, NULL);
-	unregister_trace_sys_exit(utrace_report_syscall_exit, NULL);
+	/* Unregister all the tracepoint probes. */
+	STP_TRACE_UNREGISTER(sched_process_exec, utrace_report_exec);
+	STP_TRACE_UNREGISTER(sched_process_fork, utrace_report_clone);
+	STP_TRACE_UNREGISTER(sched_process_exit, utrace_report_death);
+	STP_TRACE_UNREGISTER(sys_enter, utrace_report_syscall_entry);
+	STP_TRACE_UNREGISTER(sys_exit, utrace_report_syscall_exit);
+
+	/* When tracepoint_synchronize_unregister() returns, all
+	 * currently executing tracepoint probes will be finished. */
 	tracepoint_synchronize_unregister();
 
-	/* After calling tracepoint_synchronize_unregister(), we're
-	 * sure there are no outstanding tracepoint probes being
-	 * called.  So, now would be a great time to free everything. */
-	
+	/* (We'd like to wait here until all currrently executing
+	 * task_work items are finished (by calling
+	 * stp_task_work_exit()), but that gets stuck.)
+	 *
+	 * After the code above we're *sure* there are no tracepoint
+	 * probes running (or scheduled to be run). There could be
+	 * currently running task_work items.  Go ahead and cleanup
+	 * everything.  Currently running items should be OK, since
+	 * utrace_cleanup() just puts the memory back into the utrace
+	 * kmem caches. */
 #ifdef STP_TF_DEBUG
 	printk(KERN_ERR "%s:%d - freeing task-specific\n", __FUNCTION__, __LINE__);
 #endif
-	spin_lock(&task_utrace_lock);
+	stp_spin_lock(&task_utrace_lock);
 	for (i = 0; i < TASK_UTRACE_TABLE_SIZE; i++) {
 		head = &task_utrace_table[i];
-		stap_hlist_for_each_entry_safe(utrace, node, node2, head, hlist) {
+		stap_hlist_for_each_entry_safe(utrace, node, node2, head,
+					       hlist) {
 			hlist_del(&utrace->hlist);
 			utrace_cleanup(utrace);
 		}
 	}
-	spin_unlock(&task_utrace_lock);
+	stp_spin_unlock(&task_utrace_lock);
+#ifdef STP_TF_DEBUG
+	printk(KERN_ERR "%s:%d - done\n", __FUNCTION__, __LINE__);
+#endif
 }
 
 /*
@@ -411,14 +516,15 @@ static bool utrace_task_alloc(struct task_struct *task)
 
 	if (unlikely(!utrace))
 		return false;
-	spin_lock_init(&utrace->lock);
+	stp_spin_lock_init(&utrace->lock);
 	INIT_LIST_HEAD(&utrace->attached);
 	INIT_LIST_HEAD(&utrace->attaching);
 	utrace->resume = UTRACE_RESUME;
 	utrace->task = task;
 	stp_init_task_work(&utrace->work, &utrace_resume);
+	stp_init_task_work(&utrace->report_work, &utrace_report_work);
 
-	spin_lock(&task_utrace_lock);
+	stp_spin_lock(&task_utrace_lock);
 	u = __task_utrace_struct(task);
 	if (u == NULL) {
 		hlist_add_head(&utrace->hlist,
@@ -427,7 +533,7 @@ static bool utrace_task_alloc(struct task_struct *task)
 	else {
 		kmem_cache_free(utrace_cachep, utrace);
 	}
-	spin_unlock(&task_utrace_lock);
+	stp_spin_unlock(&task_utrace_lock);
 
 	return true;
 }
@@ -446,11 +552,12 @@ static void utrace_free(struct utrace *utrace)
 
 	/* Remove this utrace from the mapping list of tasks to
 	 * struct utrace. */
-	spin_lock(&task_utrace_lock);
+	stp_spin_lock(&task_utrace_lock);
 	hlist_del(&utrace->hlist);
-	spin_unlock(&task_utrace_lock);
+	stp_spin_unlock(&task_utrace_lock);
 
 	/* Free the utrace struct. */
+	stp_spin_lock(&utrace->lock);
 #ifdef STP_TF_DEBUG
 	if (unlikely(utrace->reporting)
 	    || unlikely(!list_empty(&utrace->attached))
@@ -470,6 +577,16 @@ static void utrace_free(struct utrace *utrace)
 				: "UNKNOWN"));
 		utrace->task_work_added = 0;
 	}
+	if (utrace->report_work_added) {
+		if (stp_task_work_cancel(utrace->task, &utrace_report_work) == NULL)
+			printk(KERN_ERR "%s:%d - task_work_cancel() failed? task %p, %d, %s\n",
+			       __FUNCTION__, __LINE__, utrace->task,
+			       utrace->task->tgid,
+			       (utrace->task->comm ? utrace->task->comm
+				: "UNKNOWN"));
+		utrace->report_work_added = 0;
+	}
+	stp_spin_unlock(&utrace->lock);
 
 	kmem_cache_free(utrace_cachep, utrace);
 }
@@ -478,9 +595,9 @@ static struct utrace *task_utrace_struct(struct task_struct *task)
 {
 	struct utrace *utrace;
 
-	spin_lock(&task_utrace_lock);
+	stp_spin_lock(&task_utrace_lock);
 	utrace = __task_utrace_struct(task);
-	spin_unlock(&task_utrace_lock);
+	stp_spin_unlock(&task_utrace_lock);
 	return utrace;
 }
 
@@ -545,7 +662,7 @@ static int utrace_add_engine(struct task_struct *target,
 {
 	int ret;
 
-	spin_lock(&utrace->lock);
+	stp_spin_lock(&utrace->lock);
 
 	ret = -EEXIST;
 	if ((flags & UTRACE_ATTACH_EXCLUSIVE) &&
@@ -593,7 +710,7 @@ static int utrace_add_engine(struct task_struct *target,
 	utrace_engine_get(engine);
 	ret = 0;
 unlock:
-	spin_unlock(&utrace->lock);
+	stp_spin_unlock(&utrace->lock);
 
 	return ret;
 }
@@ -642,11 +759,11 @@ static struct utrace_engine *utrace_attach_task(
 	if (!(flags & UTRACE_ATTACH_CREATE)) {
 		if (unlikely(!utrace))
 			return ERR_PTR(-ENOENT);
-		spin_lock(&utrace->lock);
+		stp_spin_lock(&utrace->lock);
 		engine = find_matching_engine(utrace, flags, ops, data);
 		if (engine)
 			utrace_engine_get(engine);
-		spin_unlock(&utrace->lock);
+		stp_spin_unlock(&utrace->lock);
 		return engine ?: ERR_PTR(-ENOENT);
 	}
 
@@ -762,14 +879,14 @@ static struct utrace *get_utrace_lock(struct task_struct *target,
 	}
 
 	utrace = task_utrace_struct(target);
-	spin_lock(&utrace->lock);
+	stp_spin_lock(&utrace->lock);
 	if (unlikely(utrace->reap) || unlikely(!engine->ops) ||
 	    unlikely(engine->ops == &utrace_detached_ops)) {
 		/*
 		 * By the time we got the utrace lock,
 		 * it had been reaped or detached already.
 		 */
-		spin_unlock(&utrace->lock);
+		stp_spin_unlock(&utrace->lock);
 		utrace = ERR_PTR(-ESRCH);
 		if (!attached && engine->ops == &utrace_detached_ops)
 			utrace = ERR_PTR(-ERESTARTSYS);
@@ -884,7 +1001,11 @@ static int utrace_set_events(struct task_struct *target,
 	 * If utrace_report_death() is already progress now,
 	 * it's too late to clear the death event bits.
 	 */
-	if (((old_flags & ~events) & _UTRACE_DEATH_EVENTS) && utrace->death)
+	if (target->exit_state &&
+	    (((events & ~old_flags) & _UTRACE_DEATH_EVENTS) ||
+	     (utrace->death &&
+	      ((old_flags & ~events) & _UTRACE_DEATH_EVENTS)) ||
+	     (utrace->reap && ((old_flags & ~events) & UTRACE_EVENT(REAP)))))
 		goto unlock;
 
 	/*
@@ -931,7 +1052,7 @@ static int utrace_set_events(struct task_struct *target,
 			ret = -EINPROGRESS;
 	}
 unlock:
-	spin_unlock(&utrace->lock);
+	stp_spin_unlock(&utrace->lock);
 
 	return ret;
 }
@@ -974,20 +1095,7 @@ static bool utrace_do_stop(struct task_struct *target, struct utrace *utrace)
 		spin_unlock_irq(&target->sighand->siglock);
 	} else if (utrace->resume > UTRACE_REPORT) {
 		utrace->resume = UTRACE_REPORT;
-		if (! utrace->task_work_added) {
-			int rc = stp_task_work_add(target, &utrace->work);
-			if (rc == 0) {
-				utrace->task_work_added = 1;
-			}
-			/* stp_task_work_add() returns -ESRCH if the task
-			 * has already passed exit_task_work(). Just
-			 * ignore this error. */
-			else if (rc != -ESRCH) {
-				printk(KERN_ERR
-				       "%s:%d - task_work_add() returned %d\n",
-				       __FUNCTION__, __LINE__, rc);
-			}
-		}
+		stp_task_notify_resume(target, utrace);
 	}
 
 	return task_is_traced(target);
@@ -1005,10 +1113,7 @@ static void utrace_wakeup(struct task_struct *target, struct utrace *utrace)
 	    target->signal->group_stop_count)
 		target->state = TASK_STOPPED;
 	else
-		/* FIXME: Had to change wake_up_state() to
-		 * wake_up_process() here (since wake_up_state() isn't
-		 * exported).  Reasonable? */
-		wake_up_process(target);
+		stp_wake_up_state(target, __TASK_TRACED);
 	spin_unlock_irq(&target->sighand->siglock);
 }
 
@@ -1077,7 +1182,7 @@ static bool utrace_reset(struct task_struct *task, struct utrace *utrace)
 	 */
 	rcu_read_lock();
 	utrace->utrace_flags = flags;
-	spin_unlock(&utrace->lock);
+	stp_spin_unlock(&utrace->lock);
 	rcu_read_unlock();
 
 	put_detached_list(&detached);
@@ -1093,7 +1198,7 @@ static void utrace_finish_stop(void)
 	 */
 	if (unlikely(__fatal_signal_pending(current))) {
 		struct utrace *utrace = task_utrace_struct(current);
-		spin_unlock_wait(&utrace->lock);
+		stp_spin_unlock_wait(&utrace->lock);
 	}
 }
 
@@ -1107,27 +1212,16 @@ static void utrace_stop(struct task_struct *task, struct utrace *utrace,
 			enum utrace_resume_action action)
 {
 relock:
-	spin_lock(&utrace->lock);
+	stp_spin_lock(&utrace->lock);
 
 	if (action < utrace->resume) {
 		/*
 		 * Ensure a reporting pass when we're resumed.
 		 */
 		utrace->resume = action;
-		if (! utrace->task_work_added) {
-			int rc = stp_task_work_add(task, &utrace->work);
-			if (rc == 0) {
-				utrace->task_work_added = 1;
-			}
-			/* stp_task_work_add() returns -ESRCH if the task
-			 * has already passed exit_task_work(). Just
-			 * ignore this error. */
-			else if (rc != -ESRCH) {
-				printk(KERN_ERR
-				       "%s:%d - stp_task_work_add() returned %d\n",
-				       __FUNCTION__, __LINE__, rc);
-			}
-		}
+		stp_task_notify_resume(task, utrace);
+		if (action == UTRACE_INTERRUPT)
+			set_thread_flag(TIF_SIGPENDING);
 	}
 
 	/*
@@ -1154,7 +1248,7 @@ relock:
 
 	if (unlikely(__fatal_signal_pending(task))) {
 		spin_unlock_irq(&task->sighand->siglock);
-		spin_unlock(&utrace->lock);
+		stp_spin_unlock(&utrace->lock);
 		return;
 	}
 
@@ -1169,7 +1263,7 @@ relock:
 		task->signal->flags = SIGNAL_STOP_STOPPED;
 
 	spin_unlock_irq(&task->sighand->siglock);
-	spin_unlock(&utrace->lock);
+	stp_spin_unlock(&utrace->lock);
 
 	schedule();
 
@@ -1205,7 +1299,7 @@ static void utrace_maybe_reap(struct task_struct *target, struct utrace *utrace,
 	struct utrace_engine *engine, *next;
 	struct list_head attached;
 
-	spin_lock(&utrace->lock);
+	stp_spin_lock(&utrace->lock);
 
 	if (reap) {
 		/*
@@ -1219,7 +1313,7 @@ static void utrace_maybe_reap(struct task_struct *target, struct utrace *utrace,
 		utrace->reap = 1;
 
 		if (utrace->utrace_flags & _UTRACE_DEATH_EVENTS) {
-			spin_unlock(&utrace->lock);
+			stp_spin_unlock(&utrace->lock);
 			return;
 		}
 	} else {
@@ -1257,7 +1351,7 @@ static void utrace_maybe_reap(struct task_struct *target, struct utrace *utrace,
 	list_replace_init(&utrace->attached, &attached);
 	list_splice_tail_init(&utrace->attaching, &attached);
 
-	spin_unlock(&utrace->lock);
+	stp_spin_unlock(&utrace->lock);
 
 	list_for_each_entry_safe(engine, next, &attached, entry) {
 		if (engine->flags & UTRACE_EVENT(REAP))
@@ -1432,7 +1526,7 @@ static int utrace_control(struct task_struct *target,
 	if (unlikely(target->exit_state)) {
 		ret = utrace_control_dead(target, utrace, action);
 		if (ret) {
-			spin_unlock(&utrace->lock);
+			stp_spin_unlock(&utrace->lock);
 			return ret;
 		}
 		reset = true;
@@ -1479,21 +1573,7 @@ static int utrace_control(struct task_struct *target,
 		clear_engine_wants_stop(engine);
 		if (action < utrace->resume) {
 			utrace->resume = action;
-			if (! utrace->task_work_added) {
-				ret = stp_task_work_add(target, &utrace->work);
-				if (ret == 0) {
-					utrace->task_work_added = 1;
-				}
-				/* stp_task_work_add() returns -ESRCH if
-				 * the task has already passed
-				 * exit_task_work(). Just ignore this
-				 * error. */
-				else if (ret != -ESRCH) {
-					printk(KERN_ERR
-					       "%s:%d - stp_task_work_add() returned %d\n",
-					       __FUNCTION__, __LINE__, ret);
-				}
-			}
+			stp_task_notify_resume(target, utrace);
 		}
 		break;
 
@@ -1513,6 +1593,7 @@ static int utrace_control(struct task_struct *target,
 		 * we've serialized any later recalc_sigpending() after our
 		 * setting of utrace->resume to force it on.
 		 */
+		stp_task_notify_resume(target, utrace);
 		if (reset) {
 			/*
 			 * This is really just to keep the invariant that
@@ -1522,35 +1603,12 @@ static int utrace_control(struct task_struct *target,
 			 */
 			set_tsk_thread_flag(target, TIF_SIGPENDING);
 		} else {
-			int rc = 0;
-
-			if (! utrace->task_work_added) {
-				rc = stp_task_work_add(target, &utrace->work);
-				/* stp_task_work_add() returns -ESRCH
-				 * if the task has already passed
-				 * exit_task_work(). Just ignore this
-				 * error. */
-				if (rc == 0 || rc == -ESRCH) {
-					utrace->task_work_added = 1;
-					rc = 0;
-				}
-				else {
-					printk(KERN_ERR
-					       "%s:%d - task_work_add() returned %d\n",
-					       __FUNCTION__, __LINE__, rc);
-				}
-			}
-
-			if (likely(rc == 0)) {
-				struct sighand_struct *sighand;
-				unsigned long irqflags;
-
-				sighand = stp_lock_task_sighand(target,
-								&irqflags);
-				if (likely(sighand)) {
-					stp_signal_wake_up(target, 0);
-					unlock_task_sighand(target, &irqflags);
-				}
+			struct sighand_struct *sighand;
+			unsigned long irqflags;
+			sighand = stp_lock_task_sighand(target, &irqflags);
+			if (likely(sighand)) {
+				stp_signal_wake_up(target, 0);
+				unlock_task_sighand(target, &irqflags);
 			}
 		}
 		break;
@@ -1566,7 +1624,7 @@ static int utrace_control(struct task_struct *target,
 	if (reset)
 		utrace_reset(target, utrace);
 	else
-		spin_unlock(&utrace->lock);
+		stp_spin_unlock(&utrace->lock);
 
 	return ret;
 }
@@ -1625,7 +1683,7 @@ static int utrace_barrier(struct task_struct *target,
 			 */
 			if (utrace->reporting != engine)
 				ret = 0;
-			spin_unlock(&utrace->lock);
+			stp_spin_unlock(&utrace->lock);
 			if (!ret)
 				break;
 		}
@@ -1663,12 +1721,12 @@ static enum utrace_resume_action start_report(struct utrace *utrace)
 	enum utrace_resume_action resume = utrace->resume;
 	if (utrace->pending_attach ||
 	    (resume > UTRACE_STOP && resume < UTRACE_RESUME)) {
-		spin_lock(&utrace->lock);
+		stp_spin_lock(&utrace->lock);
 		splice_attaching(utrace);
 		resume = utrace->resume;
 		if (resume > UTRACE_STOP)
 			utrace->resume = UTRACE_RESUME;
-		spin_unlock(&utrace->lock);
+		stp_spin_unlock(&utrace->lock);
 	}
 	return resume;
 }
@@ -1678,7 +1736,7 @@ static inline void finish_report_reset(struct task_struct *task,
 				       struct utrace_report *report)
 {
 	if (unlikely(report->spurious || report->detaches)) {
-		spin_lock(&utrace->lock);
+		stp_spin_lock(&utrace->lock);
 		if (utrace_reset(task, utrace))
 			report->action = UTRACE_RESUME;
 	}
@@ -1701,23 +1759,12 @@ static void finish_report(struct task_struct *task, struct utrace *utrace,
 		resume = will_not_stop ? UTRACE_REPORT : UTRACE_RESUME;
 
 	if (resume < utrace->resume) {
-		spin_lock(&utrace->lock);
+		stp_spin_lock(&utrace->lock);
 		utrace->resume = resume;
-		if (! utrace->task_work_added) {
-			int rc = stp_task_work_add(task, &utrace->work);
-			if (rc == 0) {
-				utrace->task_work_added = 1;
-			}
-			/* stp_task_work_add() returns -ESRCH if the task
-			 * has already passed exit_task_work(). Just
-			 * ignore this error. */
-			else if (rc != -ESRCH) {
-				printk(KERN_ERR
-				       "%s:%d - task_work_add() returned %d\n",
-				       __FUNCTION__, __LINE__, rc);
-			}
-		}
-		spin_unlock(&utrace->lock);
+		stp_task_notify_resume(task, utrace);
+		if (resume == UTRACE_INTERRUPT)
+			set_tsk_thread_flag(task, TIF_SIGPENDING);
+		stp_spin_unlock(&utrace->lock);
 	}
 
 	finish_report_reset(task, utrace, report);
@@ -1738,9 +1785,9 @@ static void finish_callback_report(struct task_struct *task,
 		 * This way, a 0 return is an unambiguous indicator that any
 		 * callback returning UTRACE_DETACH has indeed caused detach.
 		 */
-		spin_lock(&utrace->lock);
+		stp_spin_lock(&utrace->lock);
 		engine->ops = &utrace_detached_ops;
-		spin_unlock(&utrace->lock);
+		stp_spin_unlock(&utrace->lock);
 	}
 
 	/*
@@ -1759,16 +1806,16 @@ static void finish_callback_report(struct task_struct *task,
 			report->resume_action = action;
 
 		if (engine_wants_stop(engine)) {
-			spin_lock(&utrace->lock);
+			stp_spin_lock(&utrace->lock);
 			clear_engine_wants_stop(engine);
-			spin_unlock(&utrace->lock);
+			stp_spin_unlock(&utrace->lock);
 		}
 
 		return;
 	}
 
 	if (!engine_wants_stop(engine)) {
-		spin_lock(&utrace->lock);
+		stp_spin_lock(&utrace->lock);
 		/*
 		 * If utrace_control() came in and detached us
 		 * before we got the lock, we must not stop now.
@@ -1777,7 +1824,7 @@ static void finish_callback_report(struct task_struct *task,
 			report->detaches = true;
 		else
 			mark_engine_wants_stop(utrace, engine);
-		spin_unlock(&utrace->lock);
+		stp_spin_unlock(&utrace->lock);
 	}
 }
 
@@ -1871,9 +1918,6 @@ static const struct utrace_engine_ops *start_callback(
 			report->spurious = false;
 			return NULL;
 		}
-#ifdef STP_TF_DEBUG
-		printk(KERN_ERR "%s:%d\n", __FUNCTION__, __LINE__);
-#endif
 
 		/*
 		 * finish_callback() reset utrace->reporting after the
@@ -1885,17 +1929,11 @@ static const struct utrace_engine_ops *start_callback(
 		utrace->reporting = engine;
 		smp_mb();
 		want = engine->flags;
-#ifdef STP_TF_DEBUG
-		printk(KERN_ERR "%s:%d\n", __FUNCTION__, __LINE__);
-#endif
 	}
 
 	if (want & ENGINE_STOP)
 		report->action = UTRACE_STOP;
 
-#ifdef STP_TF_DEBUG
-	printk(KERN_ERR "%s:%d\n", __FUNCTION__, __LINE__);
-#endif
 	if (want & event) {
 		report->spurious = false;
 		return ops;
@@ -1939,7 +1977,11 @@ static void utrace_report_exec(void *cb_data __attribute__ ((unused)),
 			       pid_t old_pid __attribute__((unused)),
 			       struct linux_binprm *bprm __attribute__ ((unused)))
 {
-	struct utrace *utrace = task_utrace_struct(task);
+	struct utrace *utrace;
+
+	if (atomic_read(&utrace_state) != __UTRACE_REGISTERED)
+		return;
+	utrace = task_utrace_struct(task);
 
 	if (utrace && utrace->utrace_flags & UTRACE_EVENT(EXEC)) {
 		INIT_REPORT(report);
@@ -2004,7 +2046,11 @@ static void utrace_report_syscall_entry(void *cb_data __attribute__ ((unused)),
 					struct pt_regs *regs, long id)
 {
 	struct task_struct *task = current;
-	struct utrace *utrace = task_utrace_struct(task);
+	struct utrace *utrace;
+
+	if (atomic_read(&utrace_state) != __UTRACE_REGISTERED)
+		return;
+	utrace = task_utrace_struct(task);
 
 	/* FIXME: Is this 100% correct? */
 	if (utrace
@@ -2039,7 +2085,11 @@ static void utrace_report_syscall_exit(void *cb_data __attribute__ ((unused)),
 				       struct pt_regs *regs, long ret)
 {
 	struct task_struct *task = current;
-	struct utrace *utrace = task_utrace_struct(task);
+	struct utrace *utrace;
+
+	if (atomic_read(&utrace_state) != __UTRACE_REGISTERED)
+		return;
+	utrace = task_utrace_struct(task);
 
 	/* FIXME: Is this 100% correct? */
 	if (utrace
@@ -2066,7 +2116,11 @@ static void utrace_report_clone(void *cb_data __attribute__ ((unused)),
 				struct task_struct *task,
 				struct task_struct *child)
 {
-	struct utrace *utrace = task_utrace_struct(task);
+	struct utrace *utrace;
+
+	if (atomic_read(&utrace_state) != __UTRACE_REGISTERED)
+		return;
+	utrace = task_utrace_struct(task);
 
 #ifdef STP_TF_DEBUG
 	printk(KERN_ERR "%s:%d - parent %p, child %p, current %p\n",
@@ -2148,9 +2202,9 @@ static void utrace_finish_vfork(struct task_struct *task)
 	struct utrace *utrace = task_utrace_struct(task);
 
 	if (utrace->vfork_stop) {
-		spin_lock(&utrace->lock);
+		stp_spin_lock(&utrace->lock);
 		utrace->vfork_stop = 0;
-		spin_unlock(&utrace->lock);
+		stp_spin_unlock(&utrace->lock);
 		utrace_stop(task, utrace, UTRACE_RESUME); /* XXX */
 	}
 }
@@ -2165,9 +2219,13 @@ static void utrace_finish_vfork(struct task_struct *task)
 static void utrace_report_death(void *cb_data __attribute__ ((unused)),
 				struct task_struct *task)
 {
-	struct utrace *utrace = task_utrace_struct(task);
+	struct utrace *utrace;
 	INIT_REPORT(report);
 
+	if (atomic_read(&utrace_state) != __UTRACE_REGISTERED)
+		return;
+	utrace = task_utrace_struct(task);
+
 #ifdef STP_TF_DEBUG
 	printk(KERN_ERR "%s:%d - task %p, utrace %p, flags %lx\n", __FUNCTION__, __LINE__, task, utrace, utrace ? utrace->utrace_flags : 0);
 #endif
@@ -2197,18 +2255,44 @@ static void utrace_report_death(void *cb_data __attribute__ ((unused)),
 	 * flag and know that we are not yet fully quiescent for purposes
 	 * of detach bookkeeping.
 	 */
-	spin_lock(&utrace->lock);
-	BUG_ON(utrace->death);
-	utrace->death = 1;
-	utrace->resume = UTRACE_RESUME;
-	splice_attaching(utrace);
-	spin_unlock(&utrace->lock);
+	if (in_atomic() || irqs_disabled()) {
+		if (! utrace->report_work_added) {
+			int rc;
+#ifdef STP_TF_DEBUG
+			printk(KERN_ERR "%s:%d - adding task_work\n",
+			       __FUNCTION__, __LINE__);
+#endif
+			rc = stp_task_work_add(task,
+					       &utrace->report_work);
+			if (rc == 0) {
+				utrace->report_work_added = 1;
+			}
+			/* stp_task_work_add() returns -ESRCH if the
+			 * task has already passed
+			 * exit_task_work(). Just ignore this
+			 * error. */
+			else if (rc != -ESRCH) {
+				printk(KERN_ERR
+				       "%s:%d - task_work_add() returned %d\n",
+				       __FUNCTION__, __LINE__, rc);
+			}
+		}
+	}
+	else {
+		stp_spin_lock(&utrace->lock);
+		BUG_ON(utrace->death);
+		utrace->death = 1;
+		utrace->resume = UTRACE_RESUME;
+		splice_attaching(utrace);
+		stp_spin_unlock(&utrace->lock);
 
-	REPORT_CALLBACKS(, task, utrace, &report, UTRACE_EVENT(DEATH),
-			 report_death, engine, -1/*group_dead*/, -1/*signal*/);
+		REPORT_CALLBACKS(, task, utrace, &report, UTRACE_EVENT(DEATH),
+				 report_death, engine, -1/*group_dead*/,
+				 -1/*signal*/);
 
-	utrace_maybe_reap(task, utrace, false);
-	utrace_free(utrace);
+		utrace_maybe_reap(task, utrace, false);
+		utrace_free(utrace);
+	}
 }
 
 /*
@@ -2225,12 +2309,16 @@ static void finish_resume_report(struct task_struct *task,
 		utrace_stop(task, utrace, report->resume_action);
 		break;
 
+	case UTRACE_INTERRUPT:
+		if (!signal_pending(task)) {
+			stp_task_notify_resume(task, utrace);
+			set_tsk_thread_flag(task, TIF_SIGPENDING);
+		}
+		break;
+
 	case UTRACE_REPORT:
 	case UTRACE_RESUME:
 	default:
-#if 0
-		user_disable_single_step(task);
-#endif
 		break;
 	}
 }
@@ -2287,6 +2375,14 @@ static void utrace_resume(struct task_work *work)
 		/* Remember that this task_work_func is finished. */
 		stp_task_work_func_done();
 		return;
+	case UTRACE_INTERRUPT:
+		/*
+		 * Note that UTRACE_INTERRUPT reporting was handled by
+		 * utrace_get_signal() in original utrace. In this
+		 * utrace version, we'll handle it here like UTRACE_REPORT.
+		 *
+		 * Fallthrough...
+		 */
 	case UTRACE_REPORT:
 		if (unlikely(!(utrace->utrace_flags & UTRACE_EVENT(QUIESCE))))
 			break;
@@ -2319,4 +2415,44 @@ static void utrace_resume(struct task_work *work)
 	stp_task_work_func_done();
 }
 
+
+static void utrace_report_work(struct task_work *work)
+{
+	/*
+	 * We could also do 'task_utrace_struct()' here to find the
+	 * task's 'struct utrace', but 'container_of()' should be
+	 * instantaneous (where 'task_utrace_struct()' has to do a
+	 * hash lookup).
+	 */
+	struct utrace *utrace = container_of(work, struct utrace, report_work);
+	struct task_struct *task = current;
+	INIT_REPORT(report);
+	struct utrace_engine *engine;
+	unsigned long clone_flags;
+
+#ifdef STP_TF_DEBUG
+	printk(KERN_ERR "%s:%d - atomic %d, irqs_disabled %d\n",
+	       __FUNCTION__, __LINE__, in_atomic(), irqs_disabled());
+#endif
+	might_sleep();
+	utrace->report_work_added = 0;
+
+	stp_spin_lock(&utrace->lock);
+	BUG_ON(utrace->death);
+	utrace->death = 1;
+	utrace->resume = UTRACE_RESUME;
+	splice_attaching(utrace);
+	stp_spin_unlock(&utrace->lock);
+
+	REPORT_CALLBACKS(, task, utrace, &report, UTRACE_EVENT(DEATH),
+			 report_death, engine, -1/*group_dead*/,
+			 -1/*signal*/);
+
+	utrace_maybe_reap(task, utrace, false);
+	utrace_free(utrace);
+
+	/* Remember that this task_work_func is finished. */
+	stp_task_work_func_done();
+}
+
 #endif	/* _STP_UTRACE_C */
