@@ -35,6 +35,10 @@
 #define _STP_SYM_NEWLINE    256
 /* Adds only module " [`basename name`]" if found, use with _STP_SYM_MODULE. */
 #define _STP_SYM_MODULE_BASENAME 512
+/* Adds the line number */
+#define _STP_SYM_LINENUMBER 1024
+/* Adds the filename the symbol is from when  _STP_SYM_LINENUMBER is used. */
+#define _STP_SYM_FILENAME 2048
 
 /* Used for backtraces in hex string form. */
 #define _STP_SYM_NONE	(_STP_SYM_HEXSTR | _STP_SYM_POST_SPACE)
@@ -52,6 +56,31 @@
 #define _STP_SYM_DATA   (_STP_SYM_SYMBOL | _STP_SYM_MODULE \
 			 | _STP_SYM_OFFSET | _STP_SYM_SIZE)
 
+#ifdef STP_NEED_LINE_DATA
+/* DWARF standard opcode encodings.  */
+#define DW_LNS_copy             1
+#define DW_LNS_advance_pc       2
+#define DW_LNS_advance_line     3
+#define DW_LNS_set_file         4
+#define DW_LNS_set_column       5
+#define DW_LNS_negate_stmt      6
+#define DW_LNS_set_basic_block  7
+#define DW_LNS_const_add_pc     8
+#define DW_LNS_fixed_advance_pc 9
+#define DW_LNS_set_prologue_end 10
+#define DW_LNS_set_epilogue_begin 11
+#define DW_LNS_set_isa          12
+
+/* DWARF extended opcode encodings.  */
+#define DW_LNE_end_sequence   1
+#define DW_LNE_set_address    2
+#define DW_LNE_define_file    3
+#define DW_LNE_set_discriminator  4
+#define DW_LNE_lo_user        128
+#define DW_LNE_hi_user        255
+
+#endif /* STP_NEED_LINE_DATA */
+
 struct _stp_symbol {
 	unsigned long addr;
 	const char *symbol;
@@ -82,9 +111,11 @@ struct _stp_module {
 	void *debug_frame;
 	void *eh_frame;
 	void *unwind_hdr;	
+  void *debug_line;
 	uint32_t debug_frame_len;
 	uint32_t eh_frame_len;
 	uint32_t unwind_hdr_len;
+  uint32_t debug_line_len;
 	unsigned long eh_frame_addr; /* Orig load address (offset) .eh_frame */
 	unsigned long unwind_hdr_addr; /* same for .eh_frame_hdr */
 
@@ -115,10 +146,12 @@ static void _stp_kmodule_update_address(const char* module,
                                         const char* section,
                                         unsigned long offset);
 
-#if defined(STP_USE_DWARF_UNWINDER) && defined(STP_NEED_UNWIND_DATA)
+#if (defined(STP_USE_DWARF_UNWINDER) && defined(STP_NEED_UNWIND_DATA)) \
+    || defined(STP_NEED_LINE_DATA)
 static struct _stp_module _stp_module_self;
 static struct _stp_section _stp_module_self_sections[];
 static struct _stp_symbol _stp_module_self_symbols_0[];
 static struct _stp_symbol _stp_module_self_symbols_1[];
-#endif /* defined(STP_USE_DWARF_UNWINDER) && defined(STP_NEED_UNWIND_DATA) */
+#endif /* defined(STP_USE_DWARF_UNWINDER) && defined(STP_NEED_UNWIND_DATA)
+          || defined(STP_NEED_LINE_DATA) */
 #endif /* _STP_SYM_H_ */
