@@ -1,8 +1,8 @@
-/* -*- linux-c -*- 
+/* -*- linux-c -*-
  * transport.c - stp transport functions
  *
  * Copyright (C) IBM Corporation, 2005
- * Copyright (C) Red Hat Inc, 2005-2014
+ * Copyright (C) Red Hat Inc, 2005-2015
  * Copyright (C) Intel Corporation, 2006
  *
  * This file is part of systemtap, and is free software.  You can
@@ -21,12 +21,18 @@
 #include <linux/delay.h>
 #include <linux/mutex.h>
 #include "../uidgid_compatibility.h"
+#ifdef STAPCONF_MODULE_TRACEPOINT
+#include <trace/events/module.h>
+#include "../linux/stp_tracepoint.h"
+#endif
+
 
 static int _stp_exit_flag = 0;
 
 static uid_t _stp_uid = 0;
 static gid_t _stp_gid = 0;
 static int _stp_pid = 0;
+static int _stp_namespaces_pid = 0;
 static int _stp_remote_id = -1;
 static char _stp_remote_uri[MAXSTRINGLEN];
 
@@ -82,6 +88,11 @@ static void systemtap_module_exit(void);
 static int systemtap_module_init(void);
 
 static int _stp_module_notifier_active = 0;
+#ifdef STAPCONF_MODULE_TRACEPOINT
+/* callbacks in runtime/transport/symbols.c */
+static void _stp_module_load_tp(void *data, struct module* mod);
+static void _stp_module_free_tp(void *data, struct module* mod);
+#endif
 static int _stp_module_notifier (struct notifier_block * nb,
                                  unsigned long val, void *data);
 static struct notifier_block _stp_module_notifier_nb = {
@@ -126,7 +137,7 @@ static void _stp_handle_start(struct _stp_msg_start *st)
         // protect against excessive or premature startup
 	handle_startup = (! _stp_start_called && ! _stp_exit_called);
 	_stp_start_called = 1;
-	
+
 	if (handle_startup) {
 		dbug_trans(1, "stp_handle_start\n");
 
@@ -157,11 +168,29 @@ static void _stp_handle_start(struct _stp_msg_start *st)
                            failed: something nasty has happened, and
                            we want no further probing started.  PR16766 */
                         if (!_stp_module_notifier_active) {
-                                int rc = register_module_notifier(& _stp_module_notifier_nb);
-                                if (rc == 0)
-                                        _stp_module_notifier_active = 1;
-                                else
-                                        _stp_warn ("Cannot register module notifier (%d)\n", rc);
+#ifdef STAPCONF_MODULE_TRACEPOINT
+                                int rc0 = STP_TRACE_REGISTER(module_load, & _stp_module_load_tp);
+                                if (rc0)
+                                        _stp_warn ("Cannot register module load tracepoint (%d)\n", rc0);
+                                else {
+                                        int rc1 = STP_TRACE_REGISTER(module_free, & _stp_module_free_tp);
+                                        if (rc1) {
+                                                _stp_warn ("Cannot register module free tracepoint (%d)\n", rc1);
+                                                STP_TRACE_UNREGISTER(module_load, & _stp_module_load_tp);
+                                        } else {
+#endif
+                                                int rc = register_module_notifier(& _stp_module_notifier_nb);
+                                                if (rc == 0)
+                                                        _stp_module_notifier_active = 1;
+                                                else {
+                                                        _stp_warn ("Cannot register module notifier (%d)\n", rc);
+#ifdef STAPCONF_MODULE_TRACEPOINT
+                                                        STP_TRACE_UNREGISTER(module_load, & _stp_module_load_tp);
+                                                        STP_TRACE_UNREGISTER(module_free, & _stp_module_free_tp);
+                                                }
+                                        }
+#endif
+                                }
                         }
                 }
 
@@ -197,7 +226,12 @@ static void _stp_cleanup_and_exit(int send_exit)
 
 	        /* Unregister the module notifier. */
 	        if (_stp_module_notifier_active) {
-                        int rc = unregister_module_notifier(& _stp_module_notifier_nb);
+                        int rc;
+#ifdef STAPCONF_MODULE_TRACEPOINT
+                        STP_TRACE_UNREGISTER(module_load, & _stp_module_load_tp);
+                        STP_TRACE_UNREGISTER(module_free, & _stp_module_free_tp);
+#endif
+                        rc = unregister_module_notifier(& _stp_module_notifier_nb);
                         if (rc)
                                 _stp_warn("module_notifier unregister error %d", rc);
 	                _stp_module_notifier_active = 0;
@@ -263,6 +297,7 @@ static void _stp_detach(void)
 {
 	dbug_trans(1, "detach\n");
 	_stp_pid = 0;
+  _stp_namespaces_pid = 0;
 
 	if (!_stp_exit_flag)
 		_stp_transport_data_fs_overwrite(1);
@@ -281,6 +316,8 @@ static void _stp_attach(void)
 {
 	dbug_trans(1, "attach\n");
 	_stp_pid = current->pid;
+  if (_stp_namespaces_pid < 1)
+    _stp_namespaces_pid = _stp_pid;
 	_stp_transport_data_fs_overwrite(0);
 	init_timer(&_stp_ctl_work_timer);
 	_stp_ctl_work_timer.expires = jiffies + STP_CTL_TIMER_INTERVAL;
@@ -330,7 +367,7 @@ static void _stp_ctl_work_callback(unsigned long val)
  *	_stp_transport_close - close ctl and relayfs channels
  *
  *	This is called automatically when the module is unloaded.
- *     
+ *
  */
 static void _stp_transport_close(void)
 {
@@ -347,7 +384,7 @@ static void _stp_transport_close(void)
 
 /**
  * _stp_transport_init() is called from the module initialization.
- *   It does the bare minimum to exchange commands with staprun 
+ *   It does the bare minimum to exchange commands with staprun
  */
 static int _stp_transport_init(void)
 {
@@ -607,7 +644,7 @@ static struct dentry *_stp_get_module_dir(void)
 static int _stp_transport_fs_init(const char *module_name)
 {
 	struct dentry *root_dir;
-    
+
 	dbug_trans(1, "entry\n");
 	if (module_name == NULL)
 		return -1;
@@ -710,6 +747,12 @@ static void _stp_handle_remote_id (struct _stp_msg_remote_id* rem)
   strlcpy(_stp_remote_uri, rem->remote_uri, min(STP_REMOTE_URI_LEN,MAXSTRINGLEN));
 }
 
+static void _stp_handle_namespaces_pid (struct _stp_msg_ns_pid *nspid)
+{
+  if (nspid->target > 0)
+    _stp_namespaces_pid = (int) nspid->target;
+}
+
 
 
 #endif /* _TRANSPORT_C_ */
