@@ -22,6 +22,7 @@
 #include "cmdline.h"
 #include "git_version.h"
 #include "version.h"
+#include "stringtable.h"
 
 #include <cerrno>
 #include <cstdlib>
@@ -96,6 +97,7 @@ systemtap_session::systemtap_session ():
   suppressed_warnings(0),
   suppressed_errors(0),
   warningerr_count(0),
+  target_namespaces_pid(0),
   last_token (0)
 {
   struct utsname buf;
@@ -121,9 +123,9 @@ systemtap_session::systemtap_session ():
   dump_matched_pattern = "";
 
 #ifdef ENABLE_PROLOGUES
-  prologue_searching = true;
+  prologue_searching_mode = prologue_searching_always;
 #else
-  prologue_searching = false;
+  prologue_searching_mode = prologue_searching_auto;
 #endif
 
   buffer_size = 0;
@@ -283,6 +285,7 @@ systemtap_session::systemtap_session (const systemtap_session& other,
   suppressed_warnings(0),
   suppressed_errors(0),
   warningerr_count(0),
+  target_namespaces_pid(0),
   last_token (0)
 {
   release = kernel_release = kern;
@@ -308,7 +311,7 @@ systemtap_session::systemtap_session (const systemtap_session& other,
   dump_mode = other.dump_mode;
   dump_matched_pattern = other.dump_matched_pattern;
 
-  prologue_searching = other.prologue_searching;
+  prologue_searching_mode = other.prologue_searching_mode;
 
   buffer_size = other.buffer_size;
   last_pass = other.last_pass;
@@ -457,26 +460,11 @@ systemtap_session::version ()
 #ifdef HAVE_AVAHI
        << " AVAHI"
 #endif
-#ifdef HAVE_LIBRPM
-       << " LIBRPM"
-#endif
-#ifdef HAVE_LIBSQLITE3
-       << " LIBSQLITE3"
-#endif
-#ifdef HAVE_NSS
-       << " NSS"
-#endif
 #ifdef HAVE_BOOST_SHARED_PTR_HPP
        << " BOOST_SHARED_PTR"
 #endif
-#ifdef HAVE_TR1_UNORDERED_MAP
-       << " TR1_UNORDERED_MAP"
-#endif
-#ifdef ENABLE_PROLOGUES
-       << " PROLOGUES"
-#endif
-#ifdef ENABLE_NLS
-       << " NLS"
+#ifdef HAVE_BOOST_UTILITY_STRING_REF_HPP
+       << " BOOST_STRING_REF"
 #endif
 #ifdef HAVE_DYNINST
        << " DYNINST"
@@ -484,11 +472,29 @@ systemtap_session::version ()
 #ifdef HAVE_JAVA
        << " JAVA"
 #endif
+#ifdef HAVE_LIBRPM
+       << " LIBRPM"
+#endif
+#ifdef HAVE_LIBSQLITE3
+       << " LIBSQLITE3"
+#endif
 #ifdef HAVE_LIBVIRT
        << " LIBVIRT"
 #endif
 #ifdef HAVE_LIBXML2
        << " LIBXML2"
+#endif
+#ifdef ENABLE_NLS
+       << " NLS"
+#endif
+#ifdef HAVE_NSS
+       << " NSS"
+#endif
+#ifdef ENABLE_PROLOGUES
+       << " PROLOGUES"
+#endif
+#ifdef HAVE_TR1_UNORDERED_MAP
+       << " TR1_UNORDERED_MAP"
 #endif
        << endl;
 }
@@ -538,7 +544,7 @@ systemtap_session::usage (int exitcode)
      "   -s NUM     buffer size in megabytes, instead of %d\n"
      "   -I DIR     look in DIR for additional .stp script files", (unoptimized ? _(" [set]") : ""),
          (suppress_warnings ? _(" [set]") : ""), (panic_warnings ? _(" [set]") : ""),
-         (guru_mode ? _(" [set]") : ""), (prologue_searching ? _(" [set]") : ""),
+         (guru_mode ? _(" [set]") : ""), (prologue_searching_mode == prologue_searching_always ? _(" [set]") : ""),
          (bulk_mode ? _(" [set]") : ""), buffer_size);
   if (include_path.size() == 0)
     cout << endl;
@@ -594,6 +600,8 @@ systemtap_session::usage (int exitcode)
     "   --dyninst\n"
     "              shorthand for --runtime=dyninst\n"
 #endif /* HAVE_DYNINST */
+    "   --prologue-searching[=WHEN]\n"
+    "              prologue-searching for function probes\n"
     "   --privilege=PRIVILEGE_LEVEL\n"
     "              check the script for constructs not allowed at the given privilege level\n"
     "   --unprivileged\n"
@@ -848,7 +856,7 @@ systemtap_session::parse_cmdline (int argc, char * const argv [])
 
         case 'P':
 	  server_args.push_back (string ("-") + (char)grc);
-          prologue_searching = true;
+          prologue_searching_mode = prologue_searching_always;
           break;
 
         case 'b':
@@ -1418,6 +1426,20 @@ systemtap_session::parse_cmdline (int argc, char * const argv [])
                     strcmp(getenv("TERM") ?: "notdumb", "dumb"));
           break;
 
+        case LONG_OPT_PROLOGUE_SEARCHING:
+          // --prologue-searching without arg is equivalent to always
+          if (!optarg || !strcmp(optarg, "always"))
+            prologue_searching_mode = prologue_searching_always;
+          else if (!strcmp(optarg, "auto"))
+            prologue_searching_mode = prologue_searching_auto;
+          else if (!strcmp(optarg, "never"))
+            prologue_searching_mode = prologue_searching_never;
+          else {
+            cerr << _F("Invalid argument '%s' for --prologue-searching.", optarg) << endl;
+            return 1;
+          }
+          break;
+
         case LONG_OPT_SAVE_UPROBES:
           save_uprobes = true;
           break;
@@ -1799,11 +1821,11 @@ systemtap_session::parse_kernel_functions ()
         }
     }
 
-  string address, type, name;
   while (system_map.good())
     {
       assert_no_interrupts();
 
+      string address, type, name;
       system_map >> address >> type >> name;
 
       if (verbose > 3)
@@ -2117,7 +2139,7 @@ systemtap_session::print_error_source (std::ostream& message,
 
   unsigned line = tok->location.line;
   unsigned col = tok->location.column;
-  const string &file_contents = tok->location.file->file_contents;
+  interned_string file_contents = tok->location.file->file_contents;
 
   size_t start_pos = 0, end_pos = 0;
   //Navigate to the appropriate line
@@ -2129,7 +2151,7 @@ systemtap_session::print_error_source (std::ostream& message,
     }
   //TRANSLATORS: Here we are printing the source string of the error
   message << align << _("source: ");
-  string srcline = file_contents.substr(start_pos, end_pos-start_pos-1);
+  interned_string srcline = file_contents.substr(start_pos, end_pos-start_pos-1);
   if (color_errors) {
       // before token:
       message << srcline.substr(0, col-1);
@@ -2139,8 +2161,8 @@ systemtap_session::print_error_source (std::ostream& message,
       // or expanded from a $@ command line argument, or ...
       // so tok->content may have nothing in common with the
       // contents of srcline at the same point.
-      string srcline_rest = srcline.substr(col-1);
-      string srcline_tokenish = srcline_rest.substr(0, tok->content.size());
+      interned_string srcline_rest = srcline.substr(col-1);
+      interned_string srcline_tokenish = srcline_rest.substr(0, tok->content.size());
       if (srcline_tokenish == tok->content) { // oh good
         message << colorize(tok->content, "token");
         message << srcline_rest.substr(tok->content.size());
@@ -2258,17 +2280,17 @@ systemtap_session::build_error_msg (const parse_error& pe,
 
   if (pe.tok || found_junk)
     {
-      message << _("\tat: ") << colorize(tok) << endl;
+      message << align_parse_error << _("    at: ") << colorize(tok) << endl;
       print_error_source (message, align_parse_error, tok);
     }
   else if (tok) // "expected" type error
     {
-      message << _("\tsaw: ") << colorize(tok) << endl;
+      message << align_parse_error << _("   saw: ") << colorize(tok) << endl;
       print_error_source (message, align_parse_error, tok);
     }
   else
     {
-      message << _("\tsaw: ") << input_name << " EOF" << endl;
+      message << align_parse_error << _("   saw: ") << input_name << " EOF" << endl;
     }
   message << endl;
 
