@@ -83,11 +83,14 @@ struct be_builder: public derived_probe_builder
                      literal_map_t const & parameters,
                      vector<derived_probe *> & finished_results)
   {
-    int64_t priority;
-    if ((type == BEGIN && !get_param(parameters, TOK_BEGIN, priority)) ||
-        (type == END && !get_param(parameters, TOK_END, priority)) ||
-        (type == ERROR && !get_param(parameters, TOK_ERROR, priority)))
-      priority = 0;
+    int64_t priority = 0;
+    if (type == BEGIN)
+      get_param(parameters, TOK_BEGIN, priority);
+    else if (type == END)
+      get_param(parameters, TOK_END, priority);
+    else if (type == ERROR)
+      get_param(parameters, TOK_ERROR, priority);
+
     finished_results.push_back
       (new be_derived_probe(base, location, type, priority));
   }
@@ -100,6 +103,7 @@ be_derived_probe::join_group (systemtap_session& s)
   if (! s.be_derived_probes)
     s.be_derived_probes = new be_derived_probe_group ();
   s.be_derived_probes->enroll (this);
+  this->group = s.be_derived_probes;
 }
 
 
@@ -141,7 +145,7 @@ be_derived_probe_group::emit_module_decls (systemtap_session& s)
   common_probe_entryfn_prologue (s, "stp->state", "stp->probe",
 				 "stp_probe_type_been", false);
   s.op->newline() << "(*stp->probe->ph) (c);";
-  common_probe_entryfn_epilogue (s, false);
+  common_probe_entryfn_epilogue (s, false, otf_safe_context(s));
   s.op->newline(-1) << "}";
 }
 
