@@ -92,7 +92,7 @@ struct mark_var_expanding_visitor: public var_expanding_visitor
 void
 mark_var_expanding_visitor::visit_target_symbol_arg (target_symbol* e)
 {
-  string argnum_s = e->name.substr(4,e->name.length()-4);
+  string argnum_s = (string)e->name.substr(4,e->name.length()-4);
   int argnum = atoi (argnum_s.c_str());
 
   if (argnum < 1 || argnum > (int)mark_args.size())
@@ -401,6 +401,7 @@ mark_derived_probe::join_group (systemtap_session& s)
       s.embeds.push_back(ec);
     }
   s.mark_derived_probes->enroll (this);
+  this->group = s.mark_derived_probes;
 }
 
 
@@ -516,7 +517,7 @@ mark_derived_probe_group::emit_module_decls (systemtap_session& s)
   s.op->newline() << "(*smp->probe->ph) (c);";
   s.op->newline() << "c->ips.kmark.mark_va_list = NULL;";
 
-  common_probe_entryfn_epilogue (s, true);
+  common_probe_entryfn_epilogue (s, true, otf_safe_context(s));
   s.op->newline(-1) << "}";
 
   return;
@@ -624,9 +625,9 @@ mark_builder::build(systemtap_session & sess,
 		    literal_map_t const & parameters,
 		    vector<derived_probe *> & finished_results)
 {
-  string mark_str_val;
+  interned_string mark_str_val;
   bool has_mark_str = get_param (parameters, TOK_MARK, mark_str_val);
-  string mark_format_val;
+  interned_string mark_format_val;
   bool has_mark_format = get_param (parameters, TOK_FORMAT, mark_format_val);
   assert (has_mark_str);
   (void) has_mark_str;
@@ -724,7 +725,8 @@ mark_builder::build(systemtap_session & sess,
 	}
     }
 
-  if (results_pre == finished_results.size() && !loc->from_glob)
+  if (results_pre == finished_results.size()
+      && !loc->from_globby_comp(TOK_MARK))
     {
       string sugs = suggest_marks(sess, mark_str_val);
       if (!sugs.empty())
