@@ -5,10 +5,19 @@
 # - bdflush: obsolete since 2.6
 # - dmi_field_show: Non-syscall picked up by the "sys_*" wildcard below.
 # - dmi_modalias_show: Ditto.
+# - copyarea: Ditto.
+# - fillrect: Ditto.
+# - imageblit: Ditto.
 # - ni_syscall: non-implemented syscall
+# - rt_sigreturn: rt_sigreturn is a syscall that you don't call
+#		  directly, so we can't really test it. glibc calls it
+#		  in signal handlers. 
 # - socketcall: common entry point for other socket syscalls
 # - tux: obsolete
-set badlist { bdflush dmi_field_show dmi_modalias_show ni_syscall socketcall tux }
+set badlist {
+    bdflush dmi_field_show dmi_modalias_show copyarea fillrect
+    imageblit ni_syscall rt_sigreturn socketcall tux
+}
 
 foreach f $badlist {
     set funcname($f) -1
@@ -21,7 +30,7 @@ if {[catch {eval exec $cmd} output]} {
     exit
 }
 foreach line [split $output "\n"] {
-    if {[regexp {^kernel.function\(\"[Ss]y[Ss]_([^@]+)} $line match fn]} {
+    if {[regexp {^kernel.function\(\"[Ss]y[Ss]_([^@\"]+)} $line match fn]} {
 	if {![info exists funcname($fn)]} {
 	    set funcname($fn) 0
 	}
@@ -65,6 +74,10 @@ set handled 0
 set unhandled 0
 set unhandled_list {}
 foreach {func val} [array get funcname] {
+    if {$val < 0} {
+	continue
+    }
+
     if {$val > 0} {
 	incr covered
     } elseif {$val == 0} {
@@ -102,7 +115,7 @@ if {$i != 0} { puts "\n" }
 # Display list of handled/unhandled syscalls. A syscall is "handled"
 # if syscall/nd_syscall probes exists for the syscall.
 set total [expr $handled + $unhandled]
-puts "Handled $covered out of $total. [format "%2.1f" [expr ($handled * 100.0)/$total]]%"
+puts "Handled $handled out of $total. [format "%2.1f" [expr ($handled * 100.0)/$total]]%"
 if {$unhandled > 0} {
     puts "\nUNHANDLED FUNCTIONS"
     
