@@ -14,6 +14,9 @@
 #include <set>
 #include <iomanip>
 #include <map>
+#include <algorithm>
+#include <limits>
+
 extern "C" {
 #if ENABLE_NLS
 #include <libintl.h>
@@ -27,6 +30,7 @@ extern "C" {
 }
 
 #include "privilege.h"
+#include "stringtable.h"
 
 #if ENABLE_NLS
 #define _(string) gettext(string)
@@ -40,6 +44,11 @@ extern "C" {
 #define _F(format, ...) autosprintf(_(format), __VA_ARGS__)
 #define _NF(format, format_plural, count, ...) \
         autosprintf(_N((format), (format_plural), (count)), __VA_ARGS__)
+#define _STRINGIFY_MORE(s) #s
+#define _STRINGIFY(s) _STRINGIFY_MORE(s)
+#define ERR_SRC (std::string(__FILE__) + ":" + _STRINGIFY(__LINE__))
+#define SEMANTIC_ERROR(...) semantic_error(ERR_SRC, __VA_ARGS__)
+#define PARSE_ERROR(...) parse_error(ERR_SRC, __VA_ARGS__)
 
 const char *get_home_directory(void);
 size_t get_file_size(const std::string &path);
@@ -53,15 +62,20 @@ extern "C" gid_t get_gid (const char *group_name);
 bool in_group_id (gid_t target_gid);
 std::string getmemusage ();
 void tokenize(const std::string& str, std::vector<std::string>& tokens,
-	      const std::string& delimiters);
+	      const std::string& delimiters = " ");
 void tokenize_full(const std::string& str, std::vector<std::string>& tokens,
-	      const std::string& delimiters);
+	      const std::string& delimiters = " ");
 void tokenize_cxx(const std::string& str, std::vector<std::string>& tokens);
+std::vector<std::pair<const char*,int> > split_lines(const char *buf, size_t n);
 std::string find_executable(const std::string& name);
 std::string find_executable(const std::string& name,
 			    const std::string& sysroot,
 			    const std::map<std::string,std::string>& sysenv,
 			    const std::string& env_path = "PATH");
+bool is_fully_resolved(const std::string& name,
+                       const std::string& sysroot,
+                       const std::map<std::string,std::string>& sysenv,
+                       const std::string& env_path = "PATH");
 const std::string cmdstr_quoted(const std::string& cmd);
 const std::string cmdstr_join(const std::vector<std::string>& cmds);
 int stap_waitpid(int verbose, pid_t pid);
@@ -83,12 +97,15 @@ int regexp_match (const std::string& value, const std::string& re, std::vector<s
 bool contains_glob_chars (const std::string &str);
 std::string escape_glob_chars (const std::string& str);
 std::string unescape_glob_chars (const std::string& str);
+bool identifier_string_needs_escape (const std::string& str);
+std::string escaped_indentifier_string (const std::string& str);
 std::string kernel_release_from_build_tree (const std::string &kernel_build_tree, int verbose = 0);
 std::string normalize_machine(const std::string& machine);
 int elf_class_from_normalized_machine(const std::string& machine);
 std::string autosprintf(const char* format, ...) __attribute__ ((format (printf, 1, 2)));
 const std::set<std::string>& localization_variables();
 std::string get_self_path();
+bool is_valid_pid (pid_t pid, std::string& err_msg);
 
 // stringification generics
 
@@ -235,6 +252,11 @@ class save_and_restore
     ~save_and_restore() { *ptr = previous_value; }
 };
 
+template <typename T>
+inline bool vector_has(std::vector<T>& v, T item)
+{
+  return std::find(v.begin(), v.end(), item) != v.end();
+}
 
 // Returns whether a string starts with the given prefix
 inline bool
@@ -243,6 +265,11 @@ startswith(const std::string & s, const char * prefix)
   return (s.compare(0, std::strlen(prefix), prefix) == 0);
 }
 
+inline bool
+startswith(const std::string & s, const std::string & prefix)
+{
+  return (s.compare(0, prefix.length(), prefix) == 0);
+}
 
 // Returns whether a string ends with the given suffix
 inline bool
@@ -309,6 +336,19 @@ class Array2D
 // String sorter using the Levenshtein algorithm
 unsigned levenshtein(const std::string& a, const std::string& b);
 
+// Returns comma-separated list of set elements closest to the target string.
+// Print a maximum amount of 'max' elements, with a maximum levenshtein score
+// of 'threshold'.
+std::string levenshtein_suggest(const std::string& target,
+                                const std::set<std::string>& elems,
+                                unsigned max = std::numeric_limits<unsigned>::max(),
+                                unsigned threshold = std::numeric_limits<unsigned>::max());
+
+std::string levenshtein_suggest(const std::string& target,
+                                const std::set<interned_string>& elems,
+                                unsigned max = std::numeric_limits<unsigned>::max(),
+                                unsigned threshold = std::numeric_limits<unsigned>::max());
+
 #ifndef HAVE_PPOLL
 // This is a poor-man's ppoll; see the implementation for more details...
 int ppoll(struct pollfd *fds, nfds_t nfds,
